#!/usr/bin/env python3
##
## Authors: Adriano Marques
##          Nathan Martins
##          Thales Ribeiro
##
## Copyright (C) 2019 Exponential Ventures LLC
##
##    This library is free software; you can redistribute it and/or
##    modify it under the terms of the GNU Library General Public
##    License as published by the Free Software Foundation; either
##    version 2 of the License, or (at your option) any later version.
##
##    This library is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##    Library General Public License for more details.
##
##    You should have received a copy of the GNU Library General Public
##    License along with this library; if not, write to the Free Software
##    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
##


REPOSITORY_DIR = ".au"
KEEP_FILE = ".keep"
SOURCE_CODE_DIR = "src"
LOGS_DIR = "logs"
INITIAL_COMMIT_FILE = "initial_commit"
GITIGNORE_FILE = ".gitignore"
GITIGNORE_TEMPLATE_FILE = "gitignore_template.txt"

DATASET_METADATA_DIR = "datasets"
PARAMETER_METADATA_DIR = "parameters"
METRICS_METADATA_DIR = "metrics"
WEIGHTS_METADATA_DIR = "weights"
WEIGHTS_BINARIES_DIR = "binaries"
REQUIREMENTS_METADATA_DIR = "requirements"
CODE_METADATA_DIR = "code"
EXPERIMENTS_METADATA_DIR = "experiments"

# COMMANDS
METRICS = 'metrics'
INIT = 'init'
LOAD = 'load'
DATA = 'data'
DATA_ADD = 'add'
DATA_RM = 'rm'
EXPERIMENT_IDS = 'experiment_ids'
EXPORT_TAG = 'export'
